/*
 * Picture Puzzle Collection
 * Copyright (c) 1998-1999,2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.pp;

import java.awt.*;
import java.awt.event.*;

/**
 * Lets the user select the difficulty level of the game.  Uses mouse movement
 * to highlight the level that the player is about to select.
 * 
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: December 22, 1998
*/
public class DifficultyPanel
extends Panel
implements PuzScreen
{
// *********************
// ***** CONSTANTS *****
// *********************
	
	private static final long serialVersionUID = 8080680100L;
	
    //  static final Font LARGEFONT = new Font("Times Roman", Font.PLAIN, 36);
    static final Font LARGEFONT = new Font("Times Roman", Font.PLAIN, 64);
    
    // Text message strings
    static final String SELECT_CAPTION = "DIFFICULTY:";
    
    static final String SELECT_DIFF[] = {
        "Easy", "Normal", "Hard" };
    
    static final Color SELECT_ONCOLOR[] = {
        Color.green, Color.yellow, Color.red
    };
    
    static final Color SELECT_OFFCOLOR[] = {
        new Color(0, 128, 0), new Color(128, 128, 0), new Color(128, 0, 0)
    };

    static final int CAPTION_Y = 70;
    static final int TEXT_GAP = 90;
    static final int TEXT_START = 200;


// *********************
// ***** VARIABLES *****
// *********************

    int nCurrentSelect, nNextSelect, nSelected;
	boolean bValidRepaint;

    /**
        constructor.
    */
    public DifficultyPanel(MouseListener ml, MouseMotionListener mml)
    {
		addMouseListener(ml);
        addMouseMotionListener(mml);

		setBackground(Color.white);
        reset();
    }


    public int getSelected() {
        return nSelected;
    }
    
    
    public void paint(Graphics g) {
        if (bValidRepaint)
        bValidRepaint = false;
        else
        nCurrentSelect = -1;
        
        if (nCurrentSelect != nNextSelect) {
        int halfwidth = getSize().width / 2;
    
        nCurrentSelect = nNextSelect;
    
        Color oldColor = g.getColor();
        Font oldFont = g.getFont();
    
        g.setFont(LARGEFONT);
        FontMetrics fm = g.getFontMetrics();
        outlineText(g,
                halfwidth - (fm.stringWidth(SELECT_CAPTION) >> 1),
                CAPTION_Y, SELECT_CAPTION, Color.blue, Color.white);
    
        for (int cntr = 0; cntr < 3; ++cntr) {
            if (nCurrentSelect == (cntr + 1)) {
            outlineText(g,
                    halfwidth - (fm.stringWidth(SELECT_DIFF[cntr]) >> 1),
                    TEXT_GAP * cntr + TEXT_START,
                    SELECT_DIFF[cntr], Color.black, SELECT_ONCOLOR[cntr]);
            }
            else {
            outlineText(g,
                    halfwidth - (fm.stringWidth(SELECT_DIFF[cntr]) >> 1),
                    TEXT_GAP * cntr + TEXT_START,
                    SELECT_DIFF[cntr], Color.black, SELECT_OFFCOLOR[cntr]);
            }
        }
    
        // cleanup
        g.setColor(oldColor);
        g.setFont(oldFont);
        }
    }
    
        public void update(Graphics g) {
        bValidRepaint = true;
            paint(g);
        }
    
    /**
        Draws outlined version of text.  The font used will be the currently
        selected font.
    */
    public final void
    outlineText(Graphics g, int x, int y, String text, Color back, Color fore) {
        int cntrX, cntrY;
        Color oldColor = g.getColor();
    
        g.setColor(back);
        for (cntrX = 0; cntrX < 3; ++cntrX)
        for (cntrY = 0; cntrY < 3; ++cntrY)
            g.drawString(text, x + cntrX, y + cntrY);
    
        g.setColor(fore);
        g.drawString(text, x+1, y+1);
    
        g.setColor(oldColor);
    }
    
    
    /**
        Resets the screen to its starting condition.
    */
    public void reset() {
        nCurrentSelect = -1;
        nNextSelect = 0;
    }
    
    
    /**
        Decisecond tick notification.  The controlling object calles this function
        once every decisecond (10th of a second).  It can be used for handling
        animations and other timing dependancies.
    */
    public void tick() {
    }
    
    
    /**
        Called whenever the mouse is moved so that the screen can handle it if
        it needs to.
    */
    public void mouse_moved(int x, int y)
    {
        nNextSelect = (y - TEXT_START + TEXT_GAP * 2) / TEXT_GAP;
        if ( (nNextSelect < 0) || (nNextSelect > 3) )
        nNextSelect = 0;
        repaint();
        return;
    }

    /**
        Handle the pressing of the mouse button.  Returns a non-zero value if this
        results in an external action (such as a switch to another screen). The
        external action values are described in the MWAGConst static class.
    */
    public int mouse_released(int x, int y) {
        if ( (nCurrentSelect < 0) || (nCurrentSelect > 3) )
        return 0;
        nSelected = nCurrentSelect;
        return PuzScreen.SCREEN_GAME;
    }
    
}
