/*
 * Picture Puzzle Collection
 * Copyright (c) 1998-1999,2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.pp;

import java.awt.*;

/**
 * Displays a centered flashing ERROR see console for details message
 * 
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: December 24, 1998
*/
public class ErrorPanel
extends Panel
implements PuzScreen
{
// *********************
// ***** CONSTANTS *****
// *********************
	private static final long serialVersionUID = 8080690100L;    
    static final Font LARGEFONT = new Font("Times Roman", Font.PLAIN, 48);
    static final Font NORMFONT = new Font("Times Roman", Font.PLAIN, 24);
    
    static final String ERROR_TEXT = "PARAM ERROR";
    static final String CHECK_TEXT = "See console for details";
    

// *********************
// ***** VARIABLES *****
// *********************
    
    int nFrame;
    boolean bValidRepaint;
    String _errorText;
    String _messageText;
    
// ***********************
// ***** CONSTRUCTOR *****
// ***********************
    
  /**
    constructor.
  */
    public ErrorPanel()
    {
        setBackground(Color.white);
        _errorText = ERROR_TEXT;
        _messageText = CHECK_TEXT;
        reset();
    }
    

// *******************************
// ***** METHODS- From Panel *****
// *******************************
    
    public void paint(Graphics g)
    {
        int halfwidth = getSize().width / 2;
        int halfHeight = getSize().height / 2;
    
        Color oldColor = g.getColor();
        Font oldFont = g.getFont();
    
        // Display the error text
        g.setFont(LARGEFONT);
        if ( (nFrame % 20) < 10)
        g.setColor(Color.red);
        else
        g.setColor(Color.white);
        FontMetrics fm = g.getFontMetrics();
        g.drawString(_errorText,
            halfwidth - (fm.stringWidth(_errorText) >> 1),
            halfHeight);
    
        // Display the console text
        g.setFont(NORMFONT);
        g.setColor(Color.red);
        fm = g.getFontMetrics();
        g.drawString(_messageText,
            halfwidth - (fm.stringWidth(_messageText) >> 1),
            halfHeight + fm.getMaxAscent()*2);
    
        // cleanup
        g.setColor(oldColor);
        g.setFont(oldFont);
    }
    
        public void update(Graphics g) {
        bValidRepaint = true;
            paint(g);
        }

    
// ************************************
// ***** METHODS - From PuzScreen *****
// ************************************

    /**
        Resets the screen to its starting condition.
    */
    public void reset()
    {
        nFrame = 0;
    }
    
    
    /**
        Decisecond tick notification.  The controlling object calles this function
        once every decisecond (10th of a second).  It can be used for handling
        animations and other timing dependancies.
    */
    public void tick() {
        ++nFrame;
        repaint();
    }
    
    
    /**
        Called whenever the mouse is moved so that the screen can handle it if
        it needs to.
    */
    public void mouse_moved(int x, int y) {
        return;
    }
    
    /**
        Handle the pressing of the mouse button.  Returns a non-zero value if this
        results in an external action (such as a switch to another screen). The
        external action values are described in the MWAGConst static class.
    */
    public int mouse_released(int x, int y) {
        return 0;
    }


// *************************************
// ***** METHODS - SETING MESSAGES *****
// *************************************

    public void setError(String s)
    {
        _errorText = s;
    }
    
    public void setMessage(String s)
    {
        _messageText = s;
    }
}