/*
 * Picture Puzzle Collection
 * Copyright (c) 1998-1999,2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.pp;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import java.net.MalformedURLException;

/**
 * Sliding Puzzle Stand Alone Applet
 * 
 * @author Billy D. Spelchan
 * @version 1.0 OSR
*/
public class SPA
extends Applet
implements Runnable, MouseListener, MouseMotionListener, ActionListener {
// *** CONSTANTS ***
	private static final long serialVersionUID = 8380650100L;

// *** VARIABLES ***
  Thread runner;
  boolean threadAlive = true;
  
  SlidingPuzzle picturePuzzle;
  ErrorPanel errorPanel;
  Title title;

  PuzScreen screen;

  Image iImage;
  int piece_width, piece_height;
  Color color_correct, color_incorrect, color_select, color_slider;
  boolean grid;
  int animation;

// *** METHODS ***
	public void init() {
		this.addMouseListener(this);
    this.addMouseMotionListener(this);

    picturePuzzle = new SlidingPuzzle(this);

    if ( parseParams()) {
      screen = picturePuzzle;
        title = new Title(this);
        title.setTitle("Sliding Puzzle Applet");

	  	setLayout(new CardLayout());
        add("title", title);
        
      add("picture puzzle", picturePuzzle);
    }
    else {
      errorPanel = new ErrorPanel();
      screen = errorPanel;
	  	setLayout(new CardLayout());
      add("error", errorPanel);
    }
		requestFocus();
	}

  public boolean parseParams() {
    if (!parseImage())
      return false;

    if (!parseGrid())
      return false;

    if (!parseAnimation())
      return false;

    picturePuzzle.setPuzzle(iImage, piece_width, piece_height);
    picturePuzzle.setColor(color_correct, color_incorrect, color_select);
    picturePuzzle.setSliderColor(color_slider);
    picturePuzzle.setGrid(grid);
    picturePuzzle.setAnimation(animation);
    picturePuzzle.forceWin();
    return true;
  }

  private boolean parseImage() {
    String sTemp;

    // first find out which image to load
    sTemp = getParameter("image");
    if (sTemp != null)
      iImage = getImage(getDocumentBase(), sTemp);
    else
      iImage = getImage(getCodeBase(), "gotm/pp/pic1f.gif");

    // Now load the image (and make sure no loading error)
		MediaTracker tracker  =  new  MediaTracker(this);
    tracker.addImage(iImage, 1);
		try  {
			tracker.waitForID(1);
		}  catch  (InterruptedException  e)  {
			System.err.println("Image loading Interrupted Exception");
		}
		if (tracker.isErrorID(1)) {
			System.out.println("Image loading Error - check image parameter");
      return false;
    }

    // Now make sure image will fit within the applet frame
    if ((iImage.getWidth(this) > getSize().width) ||
          (iImage.getHeight(this) > getSize().height)) {
      System.out.println("Error - Applet size too small for image");
      return false;
    }
    return true;
  }

  private boolean parseGrid() {
    String sTemp;

    // Get the piece_width parameter (or make default)
    sTemp = getParameter("piece_width");
    if (sTemp == null)
      piece_width = 50;
    else
      piece_width = Integer.parseInt(sTemp);

    // Get the piece_height parameter (or make default)
    sTemp = getParameter("piece_height");
    if (sTemp == null)
      piece_height = piece_width;
    else
      piece_height = Integer.parseInt(sTemp);

    // Make sure parameters are acceptable
    if ( (piece_width < 5) || ((piece_width * 3) > iImage.getWidth(this)) ||
          (piece_height < 5) || ((piece_height * 3) > iImage.getHeight(this)) ){
      System.out.println("Error - piece size invalid for image");
      return false;
    }

    sTemp = getParameter("grid");
    if (sTemp == null)
      grid = false;
    else if (sTemp.equalsIgnoreCase("on"))
      grid = true;
    else
      grid = false;

    color_correct = parseColor(getParameter("color_correct"));
    if (color_correct == null)
      color_correct = Color.green;

    color_incorrect = parseColor(getParameter("color_incorrect"));
    if (color_incorrect == null)
      color_incorrect = Color.red;

    color_select = parseColor(getParameter("color_select"));
    if (color_select == null)
      color_select = Color.blue;

    color_slider = parseColor(getParameter("color_slider"));
    if (color_slider == null)
      color_slider = Color.black;

    return true;
  }

  private Color parseColor(String s) {
    if (s == null)
      return null;
    if (s.length() != 7)
      return null;

    int cntr, nUCValue;
    int nHexDigits[] = new int[6];

    for (cntr = 0; cntr < 6; ++cntr) {
      nUCValue = (int)(Character.toUpperCase(s.charAt(cntr+1)));
      if ((nUCValue >= 48) && (nUCValue <= 57))
        nHexDigits[cntr] = nUCValue - 48;
      if ((nUCValue >= 65) && (nUCValue <= 70))
        nHexDigits[cntr] = nUCValue - 55;
    }

    return new Color(nHexDigits[0]*16+nHexDigits[1],nHexDigits[2]*16+nHexDigits[3],nHexDigits[4]*16+nHexDigits[5]);
  }

  private boolean parseAnimation() {
    String sTemp;

    // Get the piece_width parameter (or make default)
    sTemp = getParameter("animation");
    if (sTemp == null)
      animation = 0;
    else {
      if (sTemp.equalsIgnoreCase("none"))
        animation = 0;
      else if (sTemp.equalsIgnoreCase("slow"))
        animation = 2;
      else if (sTemp.equalsIgnoreCase("norm"))
        animation = 4;
      else if (sTemp.equalsIgnoreCase("fast"))
        animation = 8;
      else {
        System.out.println("Error - unknown animation value");
        return false;
      }
    }
    return true;
  }


	public void start() {
		if (runner == null) {
			runner = new Thread(this);
			runner.start();
		}
	}

	public void run() {
		threadAlive = true;
		while (threadAlive) {
      screen.tick();
			try { Thread.sleep(100); }
  		catch (InterruptedException e) { }
    }
	}

	public void stop() {
		if (runner != null) {
			threadAlive = false;
			runner = null;
		}
	}

	public void destroy() {
		if (runner != null) {
			threadAlive = false;
			runner = null;
		}
    super.destroy();
	}

	public String getAppletInfo() {
    	return "Sliding Puzzle Stand Alone Applet 1.0. Copyright 1998-1999, 2003-2006 Blazing Games Inc. All Rights Reserved.";
	}


  /**
    Mouse Clicked event from MouseListener.
    Invoked when the mouse has been clicked on a component.
  */
  public void mouseClicked(MouseEvent e) { /* ignore */ }

  /**
    mousePressed event from MouseListener.
    Invoked when a mouse button has been pressed on a component.
  */
  public void mousePressed(MouseEvent e) { /* ignore */ }

 /**
    mouseReleased event from MouseListener.
    Invoked when a mouse button has been released on a component.
  */
  public void mouseReleased(MouseEvent e) {
    int ns = screen.mouse_released(e.getX(), e.getY());
    if (ns > 0) {
      picturePuzzle.scramble();
      picturePuzzle.repaint();
    }
  }

  /**
    mouseEntered event from MouseListener.
    Invoked when the mouse enters a component.
  */
  public void mouseEntered(MouseEvent e) { /* ignore */ }

 /**
    mouseExited event from MouseListener.
    Invoked when the mouse exits a component.
  */
  public void mouseExited(MouseEvent e) { /* ignore */ }

  /**
    mouseDragged event from MouseMotionListener.
    Invoked when a mouse button is pressed on a component and then dragged.
    Mouse drag events will continue to be delivered to the component where the
    first originated until the mouse button is released (regardless of whether
    the mouse position is within the bounds of the component).
  */
  public void mouseDragged(MouseEvent e) { /* ignore */ }

  /**
    mouseMoved event from MouseMotionListener.
    Invoked when the mouse button has been moved on a component (with no buttons no down).
  */
  public void mouseMoved(MouseEvent e) {
    screen.mouse_moved(e.getX(), e.getY());
  }


// ******************************************
// ***** METHODS - From Action Listener *****
// ******************************************
    public void actionPerformed(ActionEvent e)
    {
        String cmd = e.getActionCommand();
        if (cmd.equals(Title.BLAZING))
        {
            try
        {
            AppletContext ac = getAppletContext();
            URL site = new URL("http://www.blazinggames.com/index.php");
            ac.showDocument(site, "blazing");
        }
            catch (MalformedURLException notPossible) {};
        }
        else if (cmd.equals(Title.START))
        {
            ((CardLayout)getLayout()).show(this,"picture puzzle");
        }
    }
    
}
