/*
 * Picture Puzzle Collection
 * Copyright (c) 1998-1999,2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.pp;

//import java.applet.*;
import java.awt.*;
import java.awt.event.*;
//import java.awt.image.*;

/**
 * Selection Panel for selecting which picture to use as a puzzle
 * 
 * @author Billy D. Spelchan
 * @version 1.0 OSR
*/
public class SelectPanel
extends Panel
implements PuzScreen {
// *** CONSTANTS ***
	private static final long serialVersionUID = 8080830100L;
	
  static final Font LARGEFONT = new Font("Times Roman", Font.PLAIN, 34);
  static final Font NORMFONT = new Font("Times Roman", Font.PLAIN, 20);
  static final Font SMALLFONT = new Font("Times Roman", Font.PLAIN, 9);

  static final String SELECT_TEXT = "PLEASE SELECT YOUR IMAGE";
  static final String C_TEXT = "Copyright (c) 1998-1999,2003-2006 Blazing Games Inc.";
  static final int IMGPLACEX[] = {50, 370, 50, 370};
  static final int IMGPLACEY[] = {5, 5, 270, 270};

  private static final int TITLE_Y = 240;
  private static final int SELECT_Y = 265;
  private static final int C_Y = 480;
  
// *** VARIABLES ***
  String sTitle;
  Image iImages[] = new Image[4];
  int nCurrentSelect, nNextSelect, nSelected;
  boolean bValidRepaint;

  /**
    constructor.
  */
  public SelectPanel(MouseListener ml, MouseMotionListener mml) {
		addMouseListener(ml);
    addMouseMotionListener(mml);

    iImages[0] = null;
		setBackground(Color.white);
    reset();
	}

  /**
    Sets the four images the user selects from.
  */
  public void setImages(String s, Image i1, Image i2, Image i3, Image i4) {
    sTitle = s;
    iImages[0] = i1;
    iImages[1] = i2;
    iImages[2] = i3;
    iImages[3] = i4;
  }

  /**
    Returns the ID of the image selected (0-3) or -1 if none.  To be used after
    the panel has finished, most likely shortly before the actual puzzle is
    initialized.
  */
  public int getSelected() {
    return nSelected;
  }


  public void paint(Graphics g) {
    if (iImages[0] == null)
      return;

    if (bValidRepaint)
      bValidRepaint = false;
    else
      nCurrentSelect = -1;

    if (nCurrentSelect != nNextSelect) {
      int halfwidth = getSize().width / 2;
      int cntr;

      nCurrentSelect = nNextSelect;

      Color oldColor = g.getColor();
      Font oldFont = g.getFont();

      // Display the images that the user selects from.
      for (cntr = 0; cntr < 4; ++cntr) {
        if (nCurrentSelect == cntr)
          g.setColor(Color.red);
        else
          g.setColor(Color.white);
        g.drawRect(IMGPLACEX[cntr]-1,IMGPLACEY[cntr]-1, 201,201);
        g.drawRect(IMGPLACEX[cntr]-2,IMGPLACEY[cntr]-2, 203,203);
        g.drawRect(IMGPLACEX[cntr]-3,IMGPLACEY[cntr]-3, 205,205);

        g.drawImage(iImages[cntr], IMGPLACEX[cntr], IMGPLACEY[cntr], 200,200, this);
      }

      // Display the title
      g.setFont(LARGEFONT);
      FontMetrics fm = g.getFontMetrics();
      outlineText(g,
            halfwidth - (fm.stringWidth(sTitle) >> 1),
            TITLE_Y, sTitle, Color.blue, Color.white);

      // Display the select text
      g.setFont(NORMFONT);
      fm = g.getFontMetrics();
      outlineText(g,
            halfwidth - (fm.stringWidth(SELECT_TEXT) >> 1),
            SELECT_Y, SELECT_TEXT, Color.blue, Color.white);

      // Display the Copyright notice
      g.setFont(SMALLFONT);
      fm = g.getFontMetrics();
      g.setColor(Color.black);
      g.drawString(C_TEXT, halfwidth - (fm.stringWidth(C_TEXT) >> 1), C_Y);

      // cleanup
      g.setColor(oldColor);
      g.setFont(oldFont);
    }
  }

	public void update(Graphics g) {
    bValidRepaint = true;
		paint(g);
	}

  /**
    Draws outlined version of text.  The font used will be the currently
    selected font.
  */
  public final void
  outlineText(Graphics g, int x, int y, String text, Color back, Color fore) {
    int cntrX, cntrY;
    Color oldColor = g.getColor();

    g.setColor(back);
    for (cntrX = 0; cntrX < 3; ++cntrX)
      for (cntrY = 0; cntrY < 3; ++cntrY)
        g.drawString(text, x + cntrX, y + cntrY);

    g.setColor(fore);
    g.drawString(text, x+1, y+1);

    g.setColor(oldColor);
  }


  /**
    Resets the screen to its starting condition.
  */
  public void reset() {
    nCurrentSelect = -1;
    nNextSelect = 0;
    nSelected = -1;
  }


  /**
    Decisecond tick notification.  The controlling object calles this function
    once every decisecond (10th of a second).  It can be used for handling
    animations and other timing dependancies.
  */
  public void tick() {
  }


  /**
    Called whenever the mouse is moved so that the screen can handle it if
    it needs to.
  */
  public void mouse_moved(int x, int y) {
    nNextSelect = (y / 240) * 2 + x / 320;
    if ( (nNextSelect < 0) || (nNextSelect > 3))
      nNextSelect = -1;
    repaint();
    return;
  }
  /**
    Handle the pressing of the mouse button.  Returns a non-zero value if this
    results in an external action (such as a switch to another screen). The
    external action values are described in the MWAGConst static class.
  */
  public int mouse_released(int x, int y) {
    if ( (nCurrentSelect < 0) || (nCurrentSelect > 3) )
      return 0;
    nSelected = nCurrentSelect;
    return PuzScreen.SCREEN_DIFFICULTY;
  }

}