/*
 * Picture Puzzle Collection
 * Copyright (c) 1998-1999,2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.pp;

import java.awt.*;
import java.awt.event.*;
import java.util.Random;

/**
 * Sliding Puzzle game display
 * 
 * @author Billy D. Spelchan
 * @version 1.0 OSR
*/
public class SlidingPuzzle extends PicturePuzzle {

// *** CONSTANTS ***
	private static final long serialVersionUID = 8380710100L;
	
	// Slider direction constants
  public static final int SLIDE_UP = 0;
  public static final int SLIDE_RIGHT = 1;
  public static final int SLIDE_DOWN = 2;
  public static final int SLIDE_LEFT = 3;

// *** VARIABLES ***
  /** The color of the slider (empty piece) */
  Color colorSlider;

  /** The location of the slider */
  int nSliderX, nSliderY;

  /** Animation variables */
  int nAnimTop, nAnimLeft, nAnimLength, nAnimDirection;


// ***********************
// ***** CONSTRUCTOR *****
// ***********************
  
	SlidingPuzzle(MouseListener ml) {
    super(ml);

//		addMouseListener(ml);

//		setBackground(Color.white);
//    colorCorrect = Color.green;
//    colorIncorrect = Color.red;
//    colorSelect = Color.blue;
    colorSlider = Color.black;
    bGridVisible = false;

        _counterVisible = false;
        _timerVisible = false;
        _percentVisible = false;    
    }

  /** Sets the color of the slider (empty) piece */
  void setSliderColor(Color cSlider) {
    colorSlider = cSlider;
  }

  /**
    Moves slider piece in indicated direction for the indicated distance.
    If move is invalid, does nothing and returns false.
  */
  public boolean moveSlider(int direction, int distance) {
    int cntr;

    switch (direction) {
      case SLIDE_UP:
        if ((nSliderY - distance) < 0)
          return false;
        for (cntr = nSliderY; cntr > (nSliderY - distance); --cntr)
          swap(nSliderX, cntr, nSliderX, cntr-1);
        nAnimTop = nSliderY - distance;
        nAnimLeft = nSliderX;
        nSliderY -= distance;
        break;
      case SLIDE_RIGHT:
        if ((nSliderX+distance) >= nGridWidth)
          return false;
        for (cntr = nSliderX; cntr < (nSliderX + distance); ++cntr)
          swap(cntr, nSliderY, cntr+1, nSliderY);
        nAnimTop = nSliderY;
        nAnimLeft = nSliderX+1;
        nSliderX += distance;
        break;
      case SLIDE_DOWN:
        if ((nSliderY + distance) >= nGridHeight)
          return false;
        for (cntr = nSliderY; cntr < (nSliderY + distance); ++cntr)
          swap(nSliderX, cntr, nSliderX, cntr+1);
        nAnimTop = nSliderY + 1;
        nAnimLeft = nSliderX;
        nSliderY += distance;
        break;
      case SLIDE_LEFT:
        if ((nSliderX - distance) < 0)
          return false;
        for (cntr = nSliderX; cntr > (nSliderX-distance); --cntr)
          swap(cntr, nSliderY, cntr-1, nSliderY);
        nAnimTop = nSliderY;
        nAnimLeft = nSliderX - distance;
        nSliderX -= distance;
        break;
      default :
        return false;
    }
    nAnimLength = distance;
    nAnimDirection = direction;
    _counterValue -= (distance - 1);
    return true;
  }

  /**
    Mixes up the puzzle.
  */
  public void scramble() {
    int nNumMoves = nGridWidth * nGridHeight * 10;
    int cntr, moveDir;
    Random r = new Random();

    nSliderX = nGridWidth - 1;
    nSliderY = nGridHeight - 1;
    for (cntr = 0; cntr < nNumMoves; ++cntr) {
      do {
        moveDir = Math.abs(r.nextInt()) % 4;
      } while (!moveSlider(moveDir, 1));
    }
    hasWon = false;
    nCurrentSelection = -1;

    _counterValue = 0;
    _startTime = System.currentTimeMillis();
  }

  /**
    Paints the animation.
  */
  synchronized void paintAnimation(Graphics g) {
    int nNumFrames, startFrame;

    if ( (nAnimDirection == SLIDE_UP) || (nAnimDirection == SLIDE_DOWN) )
      nNumFrames = nGridPixelsY;
    else
      nNumFrames = nGridPixelsX;

    startFrame = nAnimationFrame;
    nAnimationFrame += nAnimationRate;
    if (nAnimationFrame >= nNumFrames) {
      animationPlaying = false;
      return;
    }

    int nImWidth = iPuzzle.getWidth(this);
    int nImHeight = iPuzzle.getHeight(this);
    int PicLeft = (getSize().width - nImWidth) / 2;
    int PicTop = (getSize().height -  nImHeight) / 2;
    if (_usePlayfield)
    {
        PicLeft = _playfieldRect.x;
        PicTop = _playfieldRect.y;
    }
    
    int areaLeft = PicLeft + nPicLeftGap + nAnimLeft * nGridPixelsX;
    int areaTop = PicTop + nPicTopGap + nAnimTop * nGridPixelsY;

    g.setColor(colorSlider);
    switch (nAnimDirection) {
      case SLIDE_UP:
        g.copyArea(areaLeft, areaTop + startFrame,
              nGridPixelsX, nGridPixelsY * nAnimLength, 0, nAnimationRate);
        g.fillRect(areaLeft, areaTop, nGridPixelsX, nAnimationFrame);
        break;
      case SLIDE_RIGHT:
        g.copyArea(areaLeft - startFrame, areaTop,
              nGridPixelsX * nAnimLength, nGridPixelsY, -nAnimationRate, 0);
        g.fillRect(areaLeft + nGridPixelsX * nAnimLength - nAnimationFrame,
              areaTop, nAnimationFrame, nGridPixelsY);
        break;
      case SLIDE_DOWN:
        g.copyArea(areaLeft, areaTop - startFrame,
              nGridPixelsX, nGridPixelsY * nAnimLength, 0, -nAnimationRate);
        g.fillRect(areaLeft, areaTop + nGridPixelsY * nAnimLength - nAnimationFrame,
              nGridPixelsX, nAnimationFrame);
        break;
      case SLIDE_LEFT:
        g.copyArea(areaLeft + startFrame, areaTop,
              nGridPixelsX * nAnimLength, nGridPixelsY, nAnimationRate, 0);
        g.fillRect(areaLeft, areaTop, nAnimationFrame, nGridPixelsY);
        break;
    }
  }


  /**
    paints the image and (optionally) the grid.
  */
	public void paint(Graphics g) {
    if (iPuzzle == null)
      return;

    super.paint(g);

    // Now paint the slider if the game isn't over
    if ((!hasWon) && (!animationPlaying)) {
      int nImWidth = iPuzzle.getWidth(this);
      int nImHeight = iPuzzle.getHeight(this);
      int PicLeft = (getSize().width - nImWidth) / 2;
      int PicTop = (getSize().height -  nImHeight) / 2;
      if (_usePlayfield)
      {
          PicLeft = _playfieldRect.x;
          PicTop = _playfieldRect.y;
      }
      
      g.setColor(colorSlider);
      g.fillRect(PicLeft + nPicLeftGap + nSliderX * nGridPixelsX,
            PicTop + nPicTopGap + nSliderY * nGridPixelsY,
            nGridPixelsX, nGridPixelsY);
    }
	}


  /**
    Handle the pressing of the mouse button.  Returns a non-zero value if this
    results in an external action (such as a switch to another screen). The
    external action values are described in the MWAGConst static class.
  */
  public int mouse_released(int x, int y) {
    if (animationPlaying)
      return 0;

    int gx, gy;
    int nImWidth = iPuzzle.getWidth(this);
    int nImHeight = iPuzzle.getHeight(this);
    int PicLeft = (getSize().width - nImWidth) / 2;
    int PicTop = (getSize().height -  nImHeight) / 2;
    if (_usePlayfield)
    {
        PicLeft = _playfieldRect.x;
        PicTop = _playfieldRect.y;
    }
    
    gx = (x - PicLeft - nPicLeftGap) / nGridPixelsX;
    gy = (y - PicTop - nPicTopGap) / nGridPixelsY;

    if ( (gx < 0) || (gx >= nGridWidth) || (gy < 0) || (gy >= nGridHeight) )
      return 0;
    if (hasWon)
      return PuzScreen.SCREEN_SELECTION;

    if ((gx==nSliderX)&&(gy==nSliderY))
      return 0;

    if (gx == nSliderX) {
      if (gy < nSliderY)
        moveSlider(SLIDE_UP, nSliderY - gy);
      else
        moveSlider(SLIDE_DOWN, gy - nSliderY);

      if (nAnimationRate > 0) {
        nAnimationFrame = 0;
        animationPlaying = true;
      }
    }
    else if (gy == nSliderY) {
      if (gx < nSliderX)
        moveSlider(SLIDE_LEFT, nSliderX - gx);
      else
        moveSlider(SLIDE_RIGHT, gx - nSliderX);

      if (nAnimationRate > 0) {
        nAnimationFrame = 0;
        animationPlaying = true;
      }
    }
    hasWon = isWon();

    repaint();
    return 0;
  }

}

